/*
 * File:   KNA9lumiC.c
 * Author: shima(JR1KNA)
 * Device: PIC12F1501
 * Created on 2021/06/28, 13:00
 */

#include <xc.h>
//#include <stdlib.h>
#ifndef _XTAL_FREQ
 // Unless already defined assume 4MHz system frequency
 // This definition is required to calibrate __delay_us() and __delay_ms()
 #define _XTAL_FREQ 4000000
#endif

// CONFIG1
#pragma config FOSC = INTOSC    // Oscillator Selection Bits (INTOSC oscillator: I/O function on CLKIN pin)
#pragma config WDTE = OFF       // Watchdog Timer Enable (WDT disabled)
#pragma config PWRTE = ON       // Power-up Timer Enable (PWRT enabled)
#pragma config MCLRE = OFF      // MCLR Pin Function Select (MCLR/VPP pin function is digital input)
#pragma config CP = OFF         // Flash Program Memory Code Protection (Program memory code protection is disabled)
#pragma config BOREN = OFF      // Brown-out Reset Enable (Brown-out Reset disabled)
#pragma config CLKOUTEN = OFF   // Clock Out Enable (CLKOUT function is disabled. I/O or oscillator function on the CLKOUT pin)

// CONFIG2
#pragma config WRT = OFF        // Flash Memory Self-Write Protection (Write protection off)
#pragma config STVREN = OFF     // Stack Overflow/Underflow Reset Enable (Stack Overflow or Underflow will not cause a Reset)
#pragma config BORV = LO        // Brown-out Reset Voltage Selection (Brown-out Reset Voltage (Vbor), low trip point selected.)
#pragma config LPBOR = OFF      // Low-Power Brown Out Reset (Low-Power BOR is disabled)
#pragma config LVP = OFF        // Low-Voltage Programming Enable (High-voltage on MCLR/VPP must be used for programming)

void init(void)
{
    OPTION_REG = 0b11011111;    // TMR0CS:0 = Internal instruction cycle clock (FOSC/4)
    OSCCON = 0b01101010;        //01101 =4 MHz, 01x = Internal oscillator block
    T1CON = 0b00110101;         //bit 5-4:11 = 1:8 Prescale Value, bit 2:1 = Do not synchronize external clock, bit 0:1 = Enables Timer1
    INTCON = 0b11001000;        //bit 7 GIE: Global Interrupt Enable, bit 6 PEIE: Peripheral Interrupt Enable, bit 3 IOCIE: Interrupt-on-Change Enable bit
    IOCAN = 0b00001000;         //IOCAN<5:0>: Interrupt-on-Change PORTA Negative Edge Enable bits
    PIE1 = 0b00000001;          //bit 0 TMR1IE: TMR1 Overflow Interrupt Enable bit
    ANSELA = 0b00000000;        //PORTA ANALOG SELECT REGISTER
    TRISA = 0b00001000;         // 1 = PORTA pin configured as an input (tri-stated), 0 = PORTA pin configured as an output
    PORTA = 0b00011111;         //RA<5:0>: PORTA I/O Value bits
}

unsigned char ngp,per;
unsigned short int nx,nlx,nf,vol;
void lumi_gpio(ngp,per)
{
    PORTA = PORTA | (0b00000001<<ngp);
    nlx = 2047-2047*per/100;
    while(nlx){--nlx;}
    PORTA = PORTA & ~(0b00000001<<ngp);
    nlx = 2047*per/100;
    while(nlx){--nlx;}
}
void lumi_gp0(void)
{
    RA0 = 0;
    __delay_ms(200);
    RA0 = 1;
    __delay_ms(200);
    RA0 = 0;
    __delay_ms(20);
    RA0 = 1;
    __delay_ms(100);
    RA0 = 0;
    __delay_ms(10);
    RA0 = 1;
    __delay_ms(100);
    RA0 = 0;
    __delay_ms(20);
    nx = 850;
    while(nx){
        RA0 = 1;
        __delay_us(930);
        RA0 = 0;
        __delay_us(70);
        --nx;
    }
    nx = 150;
    while(nx){
        RA0 = 1;
        __delay_us(800);
        RA0 = 0;
        __delay_us(200);
        --nx;
    }
    RA0 = 0;
}
void lamp_gp2(void)
{
    nx = 200;
    while(nx){
        nf = 200-nx;
        RA2 = 0;
        while(nf){
            --nf;
        }
        nf = nx*10;
        RA2 = 1;
        while(nf){        
            --nf;
        }
        --nx;
    }
    RA2 = 0;
}

bit MOD;
void main(void)
{
	init();
   
    RA0 = 1;
    RA1 = 1;
    RA2 = 1;
    TRISA4 = 1;
    TRISA5 = 1;
    RA4 = 0;
    RA5 = 1;
    
    if(RA3){MOD = 1;}
    
    while(MOD){
        vol = (TMR0&0b01111111);
        while(vol){lumi_gpio(1,0); --vol;}
        vol = 20;
        while(vol){lumi_gpio(1,10); --vol;}
        RA1 = 0;
        __delay_ms(20);
        RA1 = 1;
        __delay_ms(100);
        RA1 = 0;
        __delay_ms(20);
        RA1 = 1;
        __delay_ms(100);
        vol = 30;
        while(vol){lumi_gpio(1,0); --vol;}
        RA1 = 0;
        __delay_ms(20);
    }
    while(!MOD){
        vol = 80;
        while(vol){
            RA1 = 0;
            __delay_ms(20);
            RA1 = 1;
            __delay_ms(30);
            --vol;
        }
        vol = 10;
        while(vol){lumi_gpio(1,0); --vol;}
        RA1 = 1;
        __delay_ms(800);
        vol = (TMR0&0b00111111);
        while(vol){
            RA1 = 0;
            __delay_ms(20);
            RA1 = 1;
            __delay_ms(30);
            --vol;
        }
        vol = 10;
        while(vol){lumi_gpio(1,0); --vol;}
        RA1 = 1;
        __delay_ms(300);
        vol = 2;
        while(vol){
            RA1 = 0;
            __delay_ms(20);
            RA1 = 1;
            __delay_ms(30);
            --vol;
        }
        vol = 5;
        while(vol){lumi_gpio(1,0); --vol;}
        RA1 = 1;
        __delay_ms(4000);
    }
}
void interrupt isr(void)
{
    if(TMR1IF){
        RA4 = ~RA4;
        RA5 = ~RA5;
    }
    else if(!RA3&&RA2){
        RA1 = 1;
        lamp_gp2();
        lumi_gp0();
    }
    else if(!RA3){
        TRISA4 = ~TRISA4;
        TRISA5 = ~TRISA5;
        RA4 = 0;
        RA5 = 1;
    }

    IOCAF3 = 0;
    TMR1IF = 0;
}